﻿using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Auth;
using Microsoft.WindowsAzure.Storage.Blob;
using Syncfusion.DocToPDFConverter;
using Syncfusion.EJ2.DocumentEditor;
using Syncfusion.Pdf;
using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using System.Web.Http;
using System.Web.Http.Cors;
using EJ2WordDocument = Syncfusion.EJ2.DocumentEditor.WordDocument;

namespace DocumentEditorWebAPI.Controllers
{
    [EnableCors(origins: "*", headers: "*", methods: "*")]
    public class DocumentEditorController : ApiController
    {
        // Open file which send through ajax request
        [HttpPost]
        public HttpResponseMessage Import()
        {
            if (HttpContext.Current.Request.Files.Count == 0)
                return null;

            HttpPostedFile file = HttpContext.Current.Request.Files[0];
            int index = file.FileName.LastIndexOf('.');
            string type = index > -1 && index < file.FileName.Length - 1 ?
                file.FileName.Substring(index) : ".docx";
            Stream stream = file.InputStream;
            stream.Position = 0;

            EJ2WordDocument document = EJ2WordDocument.Load(stream, GetFormatType(type.ToLower()));
            string json = Newtonsoft.Json.JsonConvert.SerializeObject(document);
            document.Dispose();
            stream.Close();
            return new HttpResponseMessage() { Content = new StringContent(json) };
        }
        public string SaveAsBlob([FromBody]DocumentInfo blobInfo)
        {
            string status = "";
            try
            {
                string blobPath = blobInfo.blobPath;
                Stream documentstrem = new MemoryStream(Convert.FromBase64String(blobInfo.blobContent.Split(',')[1]));
                documentstrem.Position = 0;
                SaveDocument(blobPath, documentstrem).GetAwaiter().GetResult();
                status = "success";
            }
            catch
            {
                status = "failure";
            }
            return status;
        }
        internal async Task SaveDocument(string blobPath, Stream stream)
        {
            // Provide Azure account name and key
            StorageCredentials creds = new StorageCredentials(accountName, accountKey);
            CloudStorageAccount account = new CloudStorageAccount(creds, useHttps: true);
            CloudBlobClient client = account.CreateCloudBlobClient();
            // Provide blob name
            CloudBlobContainer container = client.GetContainerReference(blobName);
            //Get only file path from azure URL
            string filePath = blobPath.Replace("https://" + accountName + ".blob.core.windows.net/" + blobName + "/", "");
            CloudBlockBlob blob = container.GetBlockBlobReference(filePath);
            stream.Position = 0;
            await blob.UploadFromStreamAsync(stream);
        }

        /// <summary>
        /// Import blob from Azure and convert it to Sfdt format 
        /// </summary>
        /// <param name="blobPath">Azure blob URL</param>
        /// <returns></returns>
        public string ImportBlob(string blobPath)
        {
            WebClient client = new WebClient();
            //Download blob from azure path to byte array
            byte[] doc = client.DownloadData(blobPath);
            //Convert array to stream 
            Stream stream = new MemoryStream(doc);
            stream.Position = 0;
            string type = "";
            int index = blobPath.LastIndexOf('.');
            type = index > -1 && index < blobPath.Length - 1 ? blobPath.Substring(index + 1) : "";
            type = type.ToLower();
            string json = "";
            //If the file is in sfdt format we can directly open it in document editor
            if (type == "sfdt")
            {
                StreamReader reader = new StreamReader(stream);
                json = reader.ReadToEnd();
                reader.Dispose();
            }
            else
            {
                // Convert word document to Sfdt format
                WordDocument document = WordDocument.Load(stream, GetFormatType(type));
                json = Newtonsoft.Json.JsonConvert.SerializeObject(document);
                document.Dispose();
            }
            //Convert stream to Sfdt format

            stream.Dispose();
            return json;
        }


        // Import files from server with specifiec filename in Files folder
        [HttpPost]
        public HttpResponseMessage ImportFile(string fileName)
        {
            string path = HttpContext.Current.Server.MapPath("~/App_Data/" + fileName);
            if (!File.Exists(path))
                return null;
            Stream stream = File.OpenRead(path);
            string json = ImportWordDocument(stream);
            stream.Close();
            // Return mail merge template document.
            return new HttpResponseMessage() { Content = new StringContent(json) };
        }


        // Save the file in server
        [HttpPost]
        public string Save(string fileName)
        {
            try
            {
                System.Web.HttpPostedFile file = System.Web.HttpContext.Current.Request.Files[0];
                string path = System.Web.HttpContext.Current.Server.MapPath("~/App_Data/");
                Syncfusion.DocIO.DLS.WordDocument document = new Syncfusion.DocIO.DLS.WordDocument(file.InputStream);
                document.Save(path + fileName);
                document.Close();
                return "Sucess";
            }
            catch
            {
                return "Failure";
            }
        }

        [HttpPost]
        public string DocToPDF(string fileName)
        {

            if (HttpContext.Current.Request.Files.Count < 1)
            {
                return null;
            }
            System.Web.HttpPostedFile data = HttpContext.Current.Request.Files[0];
            try
            {
                //Opens a Word document.
                Syncfusion.DocIO.DLS.WordDocument wordDocument = new Syncfusion.DocIO.DLS.WordDocument(data.InputStream, Syncfusion.DocIO.FormatType.Docx);
                //Converts Word document as PDF.
                DocToPDFConverter converter = new DocToPDFConverter();
                PdfDocument pdf = converter.ConvertToPDF(wordDocument);
                MemoryStream stream = new MemoryStream();
                pdf.Save(stream);
                pdf.Close(true);
                wordDocument.Close();
                string path = System.Web.HttpContext.Current.Server.MapPath("~/App_Data/");
                FileStream fileStream = new FileStream(path + fileName, FileMode.OpenOrCreate, FileAccess.ReadWrite);
                stream.Position = 0;
                stream.CopyTo(fileStream);
                stream.Close();
                fileStream.Close();
                return "Sucess";
            }
            catch (Exception ex)
            {
                return "Failure";
            }
        }

        [HttpPost]
        public HttpResponseMessage GetFilesInfo()
        {
            string path = System.Web.HttpContext.Current.Server.MapPath("~/App_Data/");
            return new HttpResponseMessage() { Content = new StringContent(Newtonsoft.Json.JsonConvert.SerializeObject(ExplorFiles(path)), Encoding.UTF8, "text/plain") };
        }

        // GET: FileExplorer
        public object ExplorFiles(string fileDirectory)
        {
            List<Object> files = new List<Object>();
            try
            {
                foreach (string f in Directory.GetFiles(fileDirectory))
                {
                    files.Add(new { text = Path.GetFileName(f) });
                }
            }
            catch (System.Exception e)
            {
                throw new Exception("error", e);
            }
            var item = new { fileInfo = files };
            return item;
        }
        internal static Syncfusion.EJ2.DocumentEditor.FormatType GetFormatType(string format)
        {
            if (string.IsNullOrEmpty(format))
                throw new NotSupportedException("EJ2 DocumentEditor does not support this file format.");
            switch (format.ToLower())
            {
                case ".dotx":
                case ".docx":
                case ".docm":
                case ".dotm":
                    return Syncfusion.EJ2.DocumentEditor.FormatType.Docx;
                case ".dot":
                case ".doc":
                    return Syncfusion.EJ2.DocumentEditor.FormatType.Doc;
                case ".rtf":
                    return Syncfusion.EJ2.DocumentEditor.FormatType.Rtf;
                case ".txt":
                    return Syncfusion.EJ2.DocumentEditor.FormatType.Txt;
                case ".xml":
                    return Syncfusion.EJ2.DocumentEditor.FormatType.WordML;
                default:
                    throw new NotSupportedException("EJ2 DocumentEditor does not support this file format.");
            }
        }
        public string ImportWordDocument(Stream stream)
        {
            string sfdtText = "";
            EJ2WordDocument document = EJ2WordDocument.Load(stream, Syncfusion.EJ2.DocumentEditor.FormatType.Docx);
            sfdtText = Newtonsoft.Json.JsonConvert.SerializeObject(document);
            document.Dispose();
            return sfdtText;
        }
    }
    public class DocumentInfo
    {
        public string blobPath;
        public string blobContent;
    }

}
